import pytest

import core.config
import modules.contrib.dunstctl

def build_module():
    return modules.contrib.dunstctl.Module(
        config=core.config.Config([]),
        theme=None
    )

def test_load_module():
    __import__("modules.contrib.dunstctl")

def test_input_registration(mocker):
   input_register = mocker.patch('core.input.register')

   module = build_module()

   input_register.assert_called_with(
       module,
       button=core.input.LEFT_MOUSE,
       cmd=module.toggle_state
   )

def test_dunst_toggle_state(mocker):
    command = mocker.patch('util.cli.execute')

    module = build_module()

    module.toggle_state(None)
    command.assert_called_with('dunstctl set-paused toggle', ignore_errors=True)

def test_dunst_running(mocker):
    command = mocker.patch('util.cli.execute', return_value=(0, "false"))

    module = build_module()
    module.update()
    widget = module.widget()

    actual = module.state(widget)
    command.assert_called_with('dunstctl is-paused', return_exitcode=True, ignore_errors=True)
    assert actual == ['unmuted']

def test_dunst_paused(mocker):
    command = mocker.patch('util.cli.execute', return_value=(0, "true"))

    module = build_module()
    module.update()
    widget = module.widget()

    actual = module.state(widget)
    command.assert_called_with('dunstctl is-paused', return_exitcode=True, ignore_errors=True)
    assert actual == ['muted', 'warning']

def test_dunst_off(mocker):
    command = mocker.patch('util.cli.execute', return_value=(1, "dontcare"))

    module = build_module()
    module.update()
    widget = module.widget()

    actual = module.state(widget)
    command.assert_called_with('dunstctl is-paused', return_exitcode=True, ignore_errors=True)
    assert actual == ['unknown', 'critical']



# pylint: disable=C0111,R0903

"""Displays the current keyboard layout

Parameters:
    * layout.device: The device ID of the keyboard (as reported by `xinput -list`), defaults to the core device
"""

import re

import core.widget
import core.module

import util.cli

from bumblebee_status.discover import utility

class Module(core.module.Module):
    def __init__(self, config, theme):
        super().__init__(config=config, theme=theme, widgets=core.widget.Widget(self.get_layout))

        self._cmd = utility("get-kbd-layout")
        keyboard = self.parameter("device", None)
        if keyboard:
            self._cmd += " {}".format(keyboard)

    def get_layout(self, widget):
        result = util.cli.execute(self._cmd, ignore_errors=True)

        m = re.search("([a-zA-Z]+_)?([a-zA-Z]+)(\(([\w-]+)\))?", result)

        if m:
            layout = m.group(2)
            variant = m.group(3)
            return layout if not variant else "{} {}".format(layout, variant)

        return "n/a"

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



# pylint: disable=C0111,R0903

"""Toggle dunst notifications using dunstctl.

When notifications are paused using this module dunst doesn't get killed and
you'll keep getting notifications on the background that will be displayed when
unpausing. This is specially useful if you're using dunst's scripting
(https://wiki.archlinux.org/index.php/Dunst#Scripting), which requires dunst to
be running. Scripts will be executed when dunst gets unpaused.

Requires:
    * dunst v1.5.0+

contributed by `cristianmiranda <https://github.com/cristianmiranda>`_ - many thanks!
contributed by `joachimmathes <https://github.com/joachimmathes>`_ - many thanks!
"""

import core.module
import core.widget
import core.input
import util.cli


class Module(core.module.Module):
    def __init__(self, config, theme):
        super().__init__(config, theme, core.widget.Widget(""))
        core.input.register(self, button=core.input.LEFT_MOUSE, cmd=self.toggle_state)
        self.__states = {"unknown": ["unknown", "critical"],
                         "true": ["muted", "warning"],
                         "false": ["unmuted"]}

    def toggle_state(self, event):
        util.cli.execute("dunstctl set-paused toggle", ignore_errors=True)

    def state(self, widget):
        return self.__states[self.__is_dunst_paused()]

    def __is_dunst_paused(self):
        result = util.cli.execute("dunstctl is-paused",
                                  return_exitcode=True,
                                  ignore_errors=True)
        return result[1].rstrip() if result[0] == 0 else "unknown"



import pytest

pytest.importorskip("suntime")

pytest.importorskip("requests")

pytest.importorskip("dateutil.tz")

pytest.importorskip("datetime")

def test_load_module():
    __import__("modules.contrib.sun")




import pytest
from unittest import TestCase, mock

import core.config
import core.widget
import modules.contrib.network_traffic

from types import SimpleNamespace

pytest.importorskip("psutil")
pytest.importorskip("netifaces")

def io_counters_mock(recv, sent):
    return {
        'lo': SimpleNamespace(
            bytes_sent = sent,
            bytes_recv = recv
        )
    }

def gateways_response():
    return {
        'default': {
            1: ('10.0.0.10', 'lo')
        }
    }

def build_module():
    config = core.config.Config([])
    return modules.contrib.network_traffic.Module(config=config, theme=None)

def download_widget(module):
    return module.widgets()[0]

def upload_widget(module):
    return module.widgets()[1]

def mb_to_bytes(value):
    return value*1024**2

class TestNetworkTrafficUnit(TestCase):
    def test_load_module(self):
        __import__("modules.contrib.network_traffic")

    def test_initial_download_rate(self):
        module = build_module()
        assert download_widget(module).full_text() == '0.00B/s'

    def test_initial_upload_rate(self):
        module = build_module()
        assert upload_widget(module).full_text() == '0.00B/s'

    @mock.patch('netifaces.gateways')
    def test_invalid_gateways(self, gateways_mock):
        gateways_mock.return_value = { 'invalid': 'gateways' }

        module = build_module()

        assert download_widget(module).full_text() == '0.00B/s'
        assert upload_widget(module).full_text() == '0.00B/s'

    @mock.patch('psutil.net_io_counters')
    def test_invalid_io_counters(self, net_io_counters_mock):
        net_io_counters_mock.return_value = { 'invalid': 'io_counters' }

        module = build_module()

        assert download_widget(module).full_text() == '0.00B/s'
        assert upload_widget(module).full_text() == '0.00B/s'

    @mock.patch('psutil.net_io_counters')
    @mock.patch('netifaces.gateways')
    @mock.patch('netifaces.AF_INET', 1)
    def test_update_rates(self, gateways_mock, net_io_counters_mock):
        net_io_counters_mock.return_value = io_counters_mock(0, 0)
        gateways_mock.return_value = gateways_response()

        module = build_module()

        assert download_widget(module).full_text() == '0.00B/s'
        assert upload_widget(module).full_text() == '0.00B/s'

        net_io_counters_mock.return_value = io_counters_mock(
            mb_to_bytes(30),
            mb_to_bytes(0.5)
        )

        module.update()

        assert download_widget(module).full_text() == '30.00MiB/s'
        assert upload_widget(module).full_text() == '512.00KiB/s'

    def test_widget_states(self):
        module = build_module()

        assert module.state(download_widget(module)) == 'rx'
        assert module.state(upload_widget(module)) == 'tx'

    def test_invalid_widget_state(self):
        module = build_module()
        invalid_widget = core.widget.Widget(name='invalid')

        assert module.state(invalid_widget) == None



# pylint: disable=C0111,R0903
"""Displays the number of todo items from an org-mode file
Parameters:
    * todo_org.file:      File to read TODOs from (defaults to ~/org/todo.org)
    * todo_org.remaining: False by default. When true, will output the number of remaining todos instead of the number completed (i.e. 1/4 means 1 of 4 todos remaining, rather than 1 of 4 todos completed)
Based on the todo module by `codingo <https://github.com/codingo>`
"""

import re
import os.path

import core.module
import core.widget
import core.input
from util.format import asbool

class Module(core.module.Module):
    def __init__(self, config, theme):
        super().__init__(config, theme, core.widget.Widget(self.output))

        self.__todo_regex = re.compile("^\\s*\\*+\\s*TODO")
        self.__done_regex = re.compile("^\\s*\\*+\\s*DONE")

        self.__doc = os.path.expanduser(
            self.parameter("file", "~/org/todo.org")
        )
        self.__remaining = asbool(self.parameter("remaining", "False"))
        self.__todo, self.__total = self.count_items()
        core.input.register(
            self,
            button=core.input.LEFT_MOUSE,
            cmd="emacs {}".format(self.__doc)
        )

    def output(self, widget):
        if self.__remaining:
            return "TODO: {}/{}".format(self.__todo, self.__total)
        return "TODO: {}/{}".format(self.__total-self.__todo, self.__total)

    def update(self):
        self.__todo, self.__total = self.count_items()

    def count_items(self):
        todo, total = 0, 0
        try:
            with open(self.__doc, "r") as f:
                for line in f:
                    if self.__todo_regex.match(line.upper()) is not None:
                        todo += 1
                        total += 1
                    elif self.__done_regex.match(line.upper()) is not None:
                        total += 1
            return todo, total
        except OSError:
            return -1, -1

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



import sys
import pytest
from unittest.mock import Mock

import core.config

sys.modules['libvirt'] = Mock()

import modules.contrib.libvirtvms

def build_module():
    return modules.contrib.libvirtvms.Module(
        config=core.config.Config([]),
        theme=None
    )

def test_load_module():
    __import__("modules.contrib.libvirtvms")

def test_input_registration(mocker):
   input_register = mocker.patch('core.input.register')

   module = build_module()

   input_register.assert_called_with(
       module,
       button=core.input.LEFT_MOUSE,
       cmd="virt-manager"
   )

def test_status_failed(mocker):
    mocker.patch('libvirt.openReadOnly', return_value=None)

    module = build_module()
    status = module.status(None)

    assert status == "Failed to open connection to the hypervisor"

def test_status(mocker):
    virtMock = mocker.Mock()
    virtMock.numOfDomains = mocker.Mock(return_value=10)

    mocker.patch('libvirt.openReadOnly', return_value=virtMock)

    module = build_module()
    status = module.status(None)

    assert status == "VMs 10"



# pylint: disable=C0111,R0903

"""
Displays the unread emails count for one or more Thunderbird inboxes

Parameters:
    * thunderbird.home: Absolute path of your .thunderbird directory (e.g.: /home/pi/.thunderbird)
    * thunderbird.inboxes: Comma separated values for all MSF inboxes and their parent directory (account) (e.g.: imap.gmail.com/INBOX.msf,outlook.office365.com/Work.msf)

Tips:
    * You can run the following command in order to list all your Thunderbird inboxes

        find ~/.thunderbird -name '*.msf' | awk -F '/' '{print $(NF-1)"/"$(NF)}'

contributed by `cristianmiranda <https://github.com/cristianmiranda>`_ - many thanks!
"""

import core.module
import core.widget
import core.decorators
import core.input

import util.cli


class Module(core.module.Module):
    @core.decorators.every(minutes=1)
    def __init__(self, config, theme):
        super().__init__(config, theme, core.widget.Widget(self.thunderbird))

        self.__total = 0
        self.__label = ""
        self.__inboxes = []

        self.__home = self.parameter("home", "")
        inboxes = self.parameter("inboxes", "")
        if inboxes:
            self.__inboxes = util.format.aslist(inboxes)

    def thunderbird(self, _):
        return str(self.__label)

    def update(self):
        try:
            self.__total = 0
            self.__label = ""

            stream = self.__getThunderbirdStream()
            unread = self.__getUnreadMessagesByInbox(stream)

            counts = []
            for inbox in self.__inboxes:
                count = unread[inbox]
                self.__total += int(count)
                counts.append(count)

            self.__label = "/".join(counts)

        except Exception as err:
            self.__label = err

    def __getThunderbirdStream(self):
        cmd = (
            "find "
            + self.__home
            + " -name '*.msf' -exec grep -REo 'A2=[0-9]' {} + | grep"
        )
        for inbox in self.__inboxes:
            cmd += " -e {}".format(inbox)
        cmd += "| awk -F / '{print $(NF-1)\"/\"$(NF)}'"

        return util.cli.execute(cmd, shell=True).strip().split("\n")

    def __getUnreadMessagesByInbox(self, stream):
        unread = {}
        for line in stream:
            entry = line.split(":A2=")
            inbox = entry[0]
            count = entry[1]
            unread[inbox] = count

        return unread

    def state(self, widget):
        if self.__total > 0:
            return ["warning"]


# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



import pytest
from unittest import TestCase, mock

import core.config
import core.widget
import modules.contrib.uptime


def build_module():
    config = core.config.Config([])
    return modules.contrib.uptime.Module(config=config, theme=None)

def widget(module):
    return module.widgets()[0]

class UptimeTest(TestCase):
    def test_load_module(self):
        __import__("modules.contrib.uptime")

    @mock.patch('builtins.open', new_callable=mock.mock_open, read_data='300000 10.45')
    def test_uptime(self, uptime_mock):
        module = build_module()
        module.update()

        uptime_mock.assert_called_with('/proc/uptime', 'r')
        assert widget(module).full_text() == '3 days, 11:20:00'



import pytest

from util.format import *


def test_int_from_string():
    assert asint("100") == 100
    assert asint("-100") == -100
    assert asint("0") == 0


def test_int_from_none():
    assert asint(None) == 0


def test_int_from_int():
    assert asint(100) == 100
    assert asint(-100) == -100
    assert asint(0) == 0


def test_int_minimum():
    assert asint(100, minimum=10) == 100
    assert asint(100, minimum=100) == 100
    assert asint(5, minimum=10) == 10


def test_int_maximum():
    assert asint(100, maximum=200) == 100
    assert asint(100, maximum=100) == 100
    assert asint(200, maximum=100) == 100


def test_true_from_str():
    assert asbool("true") == True
    assert asbool(True) == True
    assert asbool("t") == True
    assert asbool("1") == True
    assert asbool("yes") == True
    assert asbool("y") == True
    assert asbool("on") == True


def test_false_from_str():
    assert asbool("false") == False
    assert asbool(False) == False
    assert asbool("f") == False
    assert asbool("0") == False
    assert asbool("no") == False
    assert asbool("n") == False
    assert asbool("off") == False
    assert asbool(None) == False


def test_list_from_None():
    assert aslist(None) == []


def test_list_from_list():
    assert aslist([1, 2, 3] == [1, 2, 3])


def test_list_from_str():
    assert aslist("12,13,14") == ["12", "13", "14"]


def test_byteformat():
    assert byte(500) == "500.00B"
    assert byte(1024) == "1.00KiB"
    assert byte(1024, "{:.0f}") == "1KiB"
    assert byte(1024 + 512) == "1.50KiB"
    assert byte(1024 * 1024 * 2 + 1024 * 512) == "2.50MiB"
    assert byte(1024 * 1024 * 1024 * 4 + 1024 * 1024 * 512) == "4.50GiB"
    assert byte(1024 * 1024 * 1024 * 1024 * 2) in ["2048.00GiB", "2.00TiB"]


def test_duration():
    assert duration(4 * 60 * 60 + 20 * 60) == "04:20:00"
    assert duration(4 * 60 * 60 + 20 * 60, unit=True) == "04:20:00h"
    assert duration(4 * 60 * 60 + 20 * 60, compact=True, unit=True) == "04:20h"

    assert duration(20 * 60) == "20:00"
    assert duration(20 * 60, unit=True) == "20:00m"
    assert duration(20 * 60, compact=True, unit=True) == "20:00m"

    assert duration(20) == "00:20"
    assert duration(20, unit=True) == "00:20m"
    assert duration(20, compact=True, unit=True) == "00:20m"

    assert duration(-1) == "n/a"


def test_seconds():
    assert seconds(10) == 10
    assert seconds("10") == 10

    assert seconds("5m") == 300
    assert seconds("5m20s") == 320

    assert seconds("4h") == 4 * 3600
    assert seconds("4h5m22s") == 4 * 3600 + 5 * 60 + 22

    assert seconds("4h5m") == 4 * 3600 + 5 * 60


def test_temperature():
    assert astemperature(10) == "10°C"
    assert astemperature(10, "metric") == "10°C"
    assert astemperature(-100, "imperial") == "-100°F"
    assert astemperature(-100, "kelvin") == "-100°K"


def test_temperature_case():
    assert astemperature(100, "ImPeRiAl") == "100°F"

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



# pylint: disable=C0111,R0903

"""
Displays the message that's received via unix socket.

Parameteres:
    * messagereceiver   : Unix socket address (e.g: /tmp/bumblebee_messagereceiver.sock)

Example:
    The following examples assume that /tmp/bumblebee_messagereceiver.sock is used as unix socket address.
    
    In order to send the string "I  bumblebee-status" to your status bar, use the following command: 
        echo -e '{"message":"I  bumblebee-status", "state": ""}' | socat unix-connect:/tmp/bumblebee_messagereceiver.sock STDIO

    In order to highlight the text, the state variable can be used: 
        echo -e '{"message":"I  bumblebee-status", "state": "warning"}' | socat unix-connect:/tmp/bumblebee_messagereceiver.sock STDIO

contributed by `bbernhard <https://github.com/bbernhard>`_ - many thanks!
"""

import socket
import logging
import os
import json

import core.module
import core.widget
import core.input


class Module(core.module.Module):
    @core.decorators.never
    def __init__(self, config, theme):
        super().__init__(config, theme, core.widget.Widget(self.message))

        self.background = True

        self.__unix_socket_address = self.parameter("address", "")

        self.__message = ""
        self.__state = []

    def message(self, widget):
        return self.__message

    def __read_data_from_socket(self):
        while True:
            try:
                os.unlink(self.__unix_socket_address)
            except OSError:
                if os.path.exists(self.__unix_socket_address):
                    logging.exception(
                        "Couldn't bind to unix socket %s", self.__unix_socket_address
                    )
                    raise

            with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as s:
                s.bind(self.__unix_socket_address)
                s.listen()

                conn, _ = s.accept()
                with conn:
                    while True:
                        data = conn.recv(1024)
                        if not data:
                            break
                        yield data.decode("utf-8")

    def update(self):
        try:
            for received_data in self.__read_data_from_socket():
                parsed_data = json.loads(received_data)
                self.__message = parsed_data["message"]
                self.__state = parsed_data["state"]
                core.event.trigger("update", [self.id], redraw_only=True)
        except json.JSONDecodeError:
            logging.exception("Couldn't parse message")
        except Exception:
            logging.exception("Unexpected exception while reading from socket")

    def state(self, widget):
        return self.__state


# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



from unittest import TestCase, mock
import pytest

import core.config
import core.widget
import modules.contrib.network

import socket

pytest.importorskip("netifaces")


def build_module():
    config = core.config.Config([])
    return modules.contrib.network.Module(config=config, theme=None)


def wireless_default():
    return {"default": {1: ("10.0.1.12", "wlan3")}}


def wired_default():
    return {"default": {18: ("10.0.1.12", "eth3")}}


def exec_side_effect_valid(*args, **kwargs):
    if args[0] == "iwgetid":
        return "ESSID: bumblefoo"
    if "iwconfig" in args[0]:
        return "level=-30"
    return mock.DEFAULT


def exec_side_effect_invalid(*args, **kwargs):
    return "invalid gibberish, can't parse for info"


class TestNetworkUnit(TestCase):
    def test_load_module(self):
        __import__("modules.contrib.network")

    @mock.patch("socket.create_connection")
    def test_no_internet(self, socket_mock):
        socket_mock.side_effect = Exception()
        module = build_module()
        assert module.widgets()[0].full_text() == "No connection"

    @mock.patch("util.cli.execute")
    @mock.patch("netifaces.gateways")
    @mock.patch("socket.create_connection")
    @mock.patch("netifaces.AF_INET", 1)
    @mock.patch("builtins.open", mock.mock_open(read_data="wlan3"))
    def test_valid_wireless_connection(self, socket_mock, gateways_mock, execute_mock):
        socket_mock.return_value = mock.MagicMock()
        fake_ssid = "bumblefoo"
        gateways_mock.return_value = wireless_default()
        execute_mock.side_effect = exec_side_effect_valid

        module = build_module()

        assert fake_ssid in module.widgets()[0].full_text()

    @mock.patch("netifaces.gateways")
    @mock.patch("socket.create_connection")
    @mock.patch("netifaces.AF_INET", 18)
    @mock.patch("builtins.open", mock.mock_open(read_data="wlan3"))
    def test_valid_wired_connection(self, socket_mock, gateways_mock):
        gateways_mock.return_value = wired_default()
        socket_mock.return_value = mock.MagicMock()

        module = build_module()

        assert module.widgets()[0].full_text() == "Ethernet"

    @mock.patch("netifaces.gateways")
    @mock.patch("socket.create_connection")
    def test_invalid_gateways(self, socket_mock, gateways_mock):
        socket_mock.return_value = mock.Mock()
        gateways_mock.return_value = {"xyz": "abc"}

        module = build_module()
        assert module.widgets()[0].full_text() == "No connection"

    @mock.patch("util.cli.execute")
    @mock.patch("socket.create_connection")
    @mock.patch("netifaces.gateways")
    @mock.patch("netifaces.AF_INET", 1)
    @mock.patch("builtins.open", mock.mock_open(read_data="wlan3"))
    def test_invalid_execs(self, gateways_mock, socket_mock, execute_mock):
        execute_mock.side_effect = exec_side_effect_invalid
        socket_mock.return_value = mock.MagicMock()
        gateways_mock.return_value = wireless_default()

        module = build_module()

        assert module.widgets()[0].full_text() == "Unknown ?%"

    @mock.patch("builtins.open", **{"return_value.raiseError.side_effect": Exception()})
    @mock.patch("socket.create_connection")
    @mock.patch("netifaces.gateways")
    @mock.patch("netifaces.AF_INET", 18)
    @mock.patch("builtins.open", mock.mock_open(read_data="wlan3"))
    def test_no_wireless_file(self, gateways_mock, socket_mock, mock_open):
        gateways_mock.return_value = wired_default()
        socket_mock.return_value = mock.MagicMock()
        module = build_module()

        assert module.widgets()[0].full_text() == "Ethernet"



import pytest

import util.store


@pytest.fixture
def emptyStore():
    return util.store.Store()


@pytest.fixture
def store():
    return util.store.Store()


def test_get_of_unset_key(emptyStore):
    assert emptyStore.get("any-key") == None
    assert emptyStore.get("any-key", "default-value") == "default-value"


def test_get_of_set_key(store):
    store.set("key", "value")
    assert store.get("key") == "value"


def test_overwrite_set(store):
    store.set("key", "value 1")
    store.set("key", "value 2")

    assert store.get("key") == "value 2"


def test_unused_keys(store):
    store.set("key 1", "value x")
    store.set("key 2", "value y")

    assert sorted(store.unused_keys()) == sorted(["key 1", "key 2"])

    store.get("key 2")

    assert store.unused_keys() == ["key 1"]

    store.get("key 1")

    assert store.unused_keys() == []


# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



import pytest

import util.cli


def test_valid_command():
    assert util.cli.execute("echo test") == "test\n"


def test_utf_command():
    rv = util.cli.execute("echo ÖPmŧß")
    assert util.cli.execute("echo ÖPmŧß") == "ÖPmŧß\n"


def test_invalid_command():
    with pytest.raises(RuntimeError):
        util.cli.execute("i-do-not-exist")


def test_command_exit_code():
    with pytest.raises(RuntimeError):
        util.cli.execute("cat i-do-not-exist")


def test_command_exit_code_no_error():
    util.cli.execute("cat i-do-not-exist", ignore_errors=True)


def test_async():
    assert util.cli.execute("echo test", wait=False) == ""


# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4



import pytest

import util.cli
import core.config
import modules.contrib.arch_update

@pytest.fixture
def module():
    module = modules.contrib.arch_update.Module(
        config=core.config.Config([]),
        theme=None
    )

    yield module

def test_load_module():
    __import__("modules.contrib.arch-update")

def test_load_symbolic_link_module():
    __import__("modules.contrib.arch_update")

def test_with_one_package(module, mocker):
    command = mocker.patch(
        'util.cli.execute',
        return_value=(0, 'bumblebee 1.0.0')
    )

    module.update()

    command.assert_called_with(
        'checkupdates',
        ignore_errors=True,
        return_exitcode=True
    )

    widget = module.widget()
    assert widget.full_text() == 'Update Arch: 1'
    assert module.state(widget) == None
    assert module.hidden() == False

def test_with_two_packages(module, mocker):
    command = mocker.patch(
        'util.cli.execute',
        return_value=(0, 'bumblebee 1.0.0\ni3wm 3.5.7')
    )

    module.update()

    widget = module.widget()
    assert widget.full_text() == 'Update Arch: 2'
    assert module.state(widget) == 'warning'
    assert module.hidden() == False

def test_with_no_packages(module, mocker):
    mocker.patch('util.cli.execute', return_value=(2, ''))

    module.update()

    widget = module.widget()
    assert widget.full_text() == 'Update Arch: 0'
    assert module.state(widget) == None
    assert module.hidden() == True

def test_with_unknown_code(module, mocker):
    mocker.patch('util.cli.execute', return_value=(99, 'error'))
    logger = mocker.patch('logging.error')

    module.update()

    logger.assert_called_with('checkupdates exited with {}: {}'.format(99, 'error'))

    widget = module.widget()
    assert widget.full_text() == 'Update Arch: 0'
    assert module.state(widget) == 'warning'
    assert module.hidden() == False




import pytest
from unittest import TestCase, mock

import core.config
import core.widget
import modules.contrib.publicip


def build_module():
    config = core.config.Config([])
    return modules.contrib.publicip.Module(config=config, theme=None)

def widget(module):
    return module.widgets()[0]

class PublicIPTest(TestCase):
    def test_load_module(self):
        __import__("modules.contrib.publicip")

    @mock.patch('util.location.public_ip')
    def test_public_ip(self, public_ip_mock):
        public_ip_mock.return_value = '5.12.220.2'

        module = build_module()
        module.update()

        assert widget(module).full_text() == '5.12.220.2'

    @mock.patch('util.location.public_ip')
    def test_public_ip_with_exception(self, public_ip_mock):
        public_ip_mock.side_effect = Exception

        module = build_module()
        module.update()

        assert widget(module).full_text() == 'n/a'

    def test_interval_seconds(self):
        module = build_module()

        assert module.parameter('interval') == 3600



import pytest

pytest.importorskip("tkinter")

pytest.importorskip("PIL")

pytest.importorskip("requests")

pytest.importorskip("simplejson")

def test_load_module():
    __import__("modules.contrib.octoprint")




import pytest
from unittest import TestCase, mock

import core.config
import core.widget
import modules.core.load

pytest.importorskip("os")
pytest.importorskip("multiprocessing")

def build_module():
    config = core.config.Config([])
    return modules.core.load.Module(config=config, theme=None)

def widget(module):
    return module.widgets()[0]

class TestLoad(TestCase):
    def test_load_module(self):
        __import__("modules.core.load")

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_initial_values(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 1
        load_avg_mock.return_value = (0.10, 0.20, 0.30)

        module = build_module()

        assert widget(module).full_text() == '0.00/0.00/0.00'

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_update_values(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 1
        load_avg_mock.return_value = (0.85, 0.95, 0.25)

        module = build_module()
        module.update()

        assert widget(module).full_text() == '0.85/0.95/0.25'

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_cpu_count_exception(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.side_effect = NotImplementedError
        load_avg_mock.return_value = (0.1, 0.2, 0.3)

        module = build_module()
        module.update()

        assert widget(module).full_text() == '0.10/0.20/0.30'

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_healthy_state(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 1
        load_avg_mock.return_value = (0.55, 0.95, 0.25)

        module = build_module()
        module.update()

        assert module.state(widget(module)) == None

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_warning_state(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 1
        load_avg_mock.return_value = (0.8, 0.85, 0.9)

        module = build_module()
        module.update()

        assert module.state(widget(module)) == 'warning'

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_critical_state(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 1
        load_avg_mock.return_value = (0.95, 0.85, 0.9)

        module = build_module()
        module.update()

        assert module.state(widget(module)) == 'critical'

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_healthy_state_with_8_cpus(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 8
        load_avg_mock.return_value = (4.42, 0.85, 0.9)

        module = build_module()
        module.update()

        assert module.state(widget(module)) == None

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_warning_state_with_8_cpus(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 8
        load_avg_mock.return_value = (5.65, 0.85, 0.9)

        module = build_module()
        module.update()

        assert module.state(widget(module)) == 'warning'

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    def test_critical_state_with_8_cpus(self, load_avg_mock, cpu_count_mock):
        cpu_count_mock.return_value = 8
        load_avg_mock.return_value = (6.45, 0.85, 0.9)

        module = build_module()
        module.update()

        assert module.state(widget(module)) == 'critical'

    @mock.patch('multiprocessing.cpu_count')
    @mock.patch('os.getloadavg')
    @mock.patch('core.input.register')
    def test_register_left_mouse_action(self, input_register_mock, load_avg_mock, cpu_count_mock):
        module = build_module()

        input_register_mock.assert_called_with(
            module,
            button=core.input.LEFT_MOUSE,
            cmd='gnome-system-monitor'
        )




# pylint: disable=C0111,R0903

"""Shows when a key is pressed

Parameters:
    * keys.keys: Comma-separated list of keys to monitor (defaults to "")
"""

import core.module
import core.widget
import core.decorators
import core.event

import util.format

from pynput.keyboard import Listener

NAMES = {
    "Key.cmd": "cmd",
    "Key.ctrl": "ctrl",
    "Key.shift": "shift",
    "Key.alt": "alt",
}

class Module(core.module.Module):
    @core.decorators.never
    def __init__(self, config, theme):
        super().__init__(config, theme, [])

        self._listener = Listener(on_press=self._key_press, on_release=self._key_release)

        self._keys = util.format.aslist(self.parameter("keys", "Key.cmd,Key.ctrl,Key.alt,Key.shift"))

        for k in self._keys:
            self.add_widget(name=k, full_text=self._display_name(k), hidden=True)
        self._listener.start()

    def _display_name(self, key):
        return NAMES.get(key, key)

    def _key_press(self, key):
        key = str(key)
        if not key in self._keys: return
        self.widget(key).hidden = False
        core.event.trigger("update", [self.id], redraw_only=False)

    def _key_release(self, key):
        key = str(key)
        if not key in self._keys: return
        self.widget(key).hidden = True
        core.event.trigger("update", [self.id], redraw_only=False)
    
    def state(self, widget):
        return widget.name

# vim: tabstop=8 expandtab shiftwidth=4 softtabstop=4
